import 'dart:collection';

import 'package:wedzera/src/collection/internal/abstract_iterator.dart';

typedef _IndexedElementPredicate<E> = bool Function(int index, E);

class WhereIndexedIterable<E> extends Iterable<E> {
  WhereIndexedIterable(this._iterable, this._f);

  final Iterable<E> _iterable;
  final _IndexedElementPredicate<E> _f;

  @override
  Iterator<E> get iterator => WhereIndexedIterator(_iterable.iterator, _f);
}

class WhereIndexedIterator<E> extends Iterator<E> {
  WhereIndexedIterator(this._iterator, this._f);

  final Iterator<E> _iterator;
  final _IndexedElementPredicate<E> _f;
  var _index = 0;

  @override
  bool moveNext() {
    while (_iterator.moveNext()) {
      if (_f(_index++, _iterator.current)) {
        return true;
      }
    }
    return false;
  }

  @override
  E get current => _iterator.current;
}

class DistinctIterable<T, K> extends Iterable<T> {
  DistinctIterable(this._source, this._keySelector);

  final Iterable<T> _source;
  final K Function(T) _keySelector;

  @override
  Iterator<T> get iterator => DistinctIterator(_source.iterator, _keySelector);
}

class DistinctIterator<T, K> extends AbstractIterator<T> {
  DistinctIterator(this._source, this._keySelector);

  final Iterator<T> _source;
  final K Function(T) _keySelector;
  final Set<K> _observed = HashSet<K>();

  @override
  void computeNext() {
    while (_source.moveNext()) {
      final next = _source.current;
      final key = _keySelector(next);

      if (_observed.add(key)) {
        setNext(next);
        return;
      }
    }
    done();
  }
}
